(************************************************************************
 Author:	Robert Rossmair
		Wagelwitz, Rodaer Str. 6
		04688 Mutzschen
		Germany

       e-mail:	Robert.Rossmair@t-online.de

 Module:	RRMath

 Version:       1.0.2, 21-Aug-1998

 History:
   28-Mar-1999: version 1.0.3
     changed license to freeware
   21-Aug-1998: version 1.0.2
     removed sTan function; it was useful on a sytem w/o FPU, but
     nowadays there aren't lots of these around any longer, right?
   24-Jun-1998: version 1.0.1
     Changed exp2 argument and result to Extended (was Single before)
     Renamed tan to sTan to avoid a naming conflict with Math.Tan
   02-Oct-1997: version 1.0

 Compatibility: Delphi 2 & 3

 Description:	32-bit (not at all!) math routines

 Copyright  1997-1999  Robert Rossmair

 Use, modification and distribution is allowed without limitation,
 warranty, or liability of any kind. If you use this software or portions
 of it in a commercial program, I would appreciate it if you mentioned my
 contribution in the documentation materials accompaning your work.
 ************************************************************************)
unit RRMath;

interface

uses
  Windows, SysUtils;

type
  NaturalNumber = 1..High(Integer);
  { Bruch }
  IntFraction = record
    Numerator:	 Smallint;
    Denominator: Smallint;
  end;

  WordFraction = record
    Numerator:	 word;
    Denominator: word;
  end;

  Coordinates = (Cartesian, Polar);
  TComplex = record
    case Coordinates of
    Cartesian: (
	Re: single;
	Im: single);
    Polar: (
	Amp:	single;
	Phase:	single);
  end;
  PComplex = ^TComplex;

  PComplexL = ^TComplexL;
  TComplexL = record
    case Coordinates of
    Cartesian: (
	Re: Longint;
	Im: Longint);
    Polar: (
	Amp:	Longint;
	Phase:	Longint);
  end;

  T3DVector = array[0..2] of Double;

const
{ FPU precision control }
  prSingle	= $0000;
  prDouble	= $0002;
  prExtended	= $0003;
{ Konstanten fuer FPU-Exceptions (Status- und Kontrollregister) }
  exInvOp	= $0001;
  exDenorm	= $0002;
  exZeroDiv	= $0004;
  exOverflow	= $0008;
  exUnderflow	= $0010;
  exPrecision	= $0020;
  exStackFault	= $0040;
  exErrorSum	= $0080;
{ Konstanten fuer RC-Feld des FPU - Kontrollregisters: }
  rcNearOrEven	= $0000;
  rcDown	= $0400;
  rcUp		= $0800;
  rcChop	= $0C00;
{ FPU - Statusregister }
  stC0		= $0100;
  stC1		= $0200;
  stC2		= $0400;
  stC3		= $4000;
  stTOP		= $3800;
  stBusy	= $8000;

  RC_Mask = $0C00;

(* Constants rounded for 21 decimals. *)
  M_E		= 2.71828182845904523536;
  M_LOG2E	= 1.44269504088896340736;
  M_LOG10E	= 0.434294481903251827651;
  M_LN2		= 0.693147180559945309417;
  M_LN10	= 2.30258509299404568402;
  M_PI		= 3.14159265358979323846;
  M_PI_2	= 1.57079632679489661923;
  M_PI_4	= 0.785398163397448309116;
  M_1_PI	= 0.318309886183790671538;
  M_2_PI	= 0.636619772367581343076;
  M_1_SQRTPI	= 0.564189583547756286948;
  M_2_SQRTPI	= 1.12837916709551257390;
  M_SQRT2	= 1.41421356237309504880;
  M_SQRT_2	= 0.707106781186547524401;
  ld_e		= 1.44269504088896E+0000;	{ ld(e)  = 1/ln(2) }
  ld_10		= 3.32192809488736E+0000;	{ ld(10) = 1/lg(2) }
  ln_2		= 6.93147180559945E-0001;
  lg_2		= 3.01029995663981E-0001;	{ lg(2) = 1/ld(10) }
  divSqrt2Pi	= 3.98942280401433E-0001;	{ 1/Sqrt(2*Pi) }
  TwoExp31	= 2.147483648E+0009;
  TwoExp32	= 4.294967296E+0009;
type
  TComplexArray = array[Word] of TComplex;
  PComplexArray = ^TComplexArray;

  TComplexLArray = array[Word] of TComplexL;
  PComplexLArray = ^TComplexLArray;
  TFixedArray = array[Word] of TFixed;
  PFixedArray = ^TFixedArray;
  PFixed = ^TFixed;

function Average(X, Y: Integer): Integer; register;
function BitCount(X: Integer): Integer;		{ binaere Quersumme }
function ClipInteger(X: Integer): Smallint;
procedure ConvCartesian(var Z: TComplex);
procedure ConvPolar(var Z: TComplex);
function exp2(X: Extended): Extended;	{ 2^X }
function Factorial(X: word): extended;	{ X!  }
procedure FPU_RoundMethod(Method: word);
function I0(X: single): single; { berechnet modifizierte Besselfunktion 1. Gattung, 0. Ordnung }
//  Interpolate?? berechnen die Funktion
//
//	y := y1 + (y2 - y1) * (x - x1) / (x2 - x1)
function Interpolate(X, X1, Y1, X2, Y2: Integer): Integer;
function Interpolate_F(X, X1, Y1, X2, Y2: Single): Single;
{$IFDEF NoMath}
function Ceil(X: Extended): Integer;
function Floor(X: Extended): Integer;
function Log2(X: Extended): Extended;                      { Log base 2 of X }
function Log10(X: Extended): Extended;                     { Log base 10 of X}
{$ENDIF}
function Long(X: single): longint;	{ Round(Min(Max(X, LongMin), LongMax) }

function IMulDiv(Factor1, Factor2, Divisor: Integer): Integer; register;
function UMulDiv(Factor1, Factor2, Divisor: UInt): UInt; register;
function IMax(Arg1, Arg2: Integer): Integer; register;
function IMin(Arg1, Arg2: Integer): Integer; register;
function UMax(Arg1, Arg2: UInt): UInt; register;
function UMin(Arg1, Arg2: UInt): UInt; register;
function MaxS(Arg1, Arg2: Single): Single;
function MaxD(Arg1, Arg2: Double): Double;
function Max(Arg1, Arg2: Extended): Extended;
function MinS(Arg1, Arg2: Single): Single;
function MinD(Arg1, Arg2: Double): Double;
function Min(Arg1, Arg2: Extended): Extended;
procedure MulComplex(Z1, Z2: TComplex; var Result: TComplex);
procedure DivComplex(Z1, Z2: TComplex; var Result: TComplex);
function NextMultipleTo(X, Y: Integer): Integer; register;
function NormDistribution(rx, ru, rs: single): single;
function SqrFixed(X: TFixed): TFixed;
function Sign(X: Integer): Integer; register;

implementation

// uses ExpData;

function Average(X, Y: Integer): Integer; assembler;
asm
	add	eax, edx
	jo	@1
	sar	eax, 1
	jmp	@2
@1:	rcr	eax, 1
@2:	adc	eax, 0
end;

function IMulDiv(Factor1, Factor2, Divisor: Integer): Integer;
asm
	imul   edx
	idiv   ecx
end;

function UMulDiv(Factor1, Factor2, Divisor: UInt): UInt;
asm
	mul   edx
	div   ecx
end;

function IMax(Arg1, Arg2: Integer): Integer;
asm
	cmp	edx,eax
	jng	@1
	mov	eax,edx
@1:
end;

function IMin(Arg1, Arg2: Integer): Integer;
asm
	cmp	eax,edx
	jng	@1
	mov	eax,edx
@1:
end;

function UMin(Arg1, Arg2: UInt): UInt;
asm
	cmp	eax, edx
	jna	@@1
	mov	eax, edx
@@1:
end;

function UMax(Arg1, Arg2: UInt): UInt;
asm
	cmp	edx, eax
	jna	@@1
	mov	eax, edx
@@1:
end;

function BitCount(X: Integer): Integer;
asm
	mov	edx, eax
	xor	eax, eax
	mov	ecx, 32
@1:	shr	edx, 1
	adc	eax, 0
	loop	@1
end;

function ClipInteger(X: Integer): Smallint;
asm
	cmp	eax, 32767
	jng	@1
	mov	ax, 32767
	ret
@1:	cmp	eax, -32768
	jge	@exit
	mov	ax, -32768
@exit:
end;

procedure MulComplex(Z1, Z2: TComplex; var Result: TComplex);
begin
  Result.Re := Z1.Re * Z2.Re - Z1.Im * Z2.Im;
  Result.Im := Z1.Re * Z2.Im + Z1.Im * Z2.Re;
end;

procedure DivComplex(Z1, Z2: TComplex; var Result: TComplex);
var
  D: single;
begin
  D := Sqr(Z2.Re) + Sqr(Z2.Im);
  Result.Re := (Z1.Re * Z2.Re + Z1.Im * Z2.Im) / D;
  Result.Im := (Z1.Re * Z2.Im - Z1.Im * Z2.Re) / D;
end;

procedure ConvCartesian(var Z: TComplex);
var
  R_: single;
begin
  with Z do
  begin
    R_ := cos(Phase) * Amp;
    Im := sin(Phase) * Amp;
    Re := R_;
  end;
end;

procedure ConvPolar(var Z: TComplex);
var
  A: single;
begin
  with Z do
  begin
    A := Sqrt(Sqr(Re) + Sqr(Im));
    if Re = 0 then
      if Im = 0	then Phase := 0 else
      if Im > 0	then Phase := 0.5 * Pi
		else Phase :=-0.5 * Pi else
    if Re > 0	then Phase := ArcTan(Im/Re)
		else Phase := ArcTan(Im/Re)+Pi;
    Amp := A;
  end;
end;

function exp2(X: Extended): Extended;
asm
	FLD	X
	FLD     ST(0)       { i := round(x);     }
	FRNDINT
	FSUB    ST(1), ST   { f := x - i;        }
	FXCH    ST(1)       { y := 2**f          }
	F2XM1
	FLD1
	FADD
	FSCALE              { result := y * 2**i }
	FSTP    ST(1)
end;
(*
function exp2(X: Extended): Extended;
{ Wichtig: im RC - Feld muss rcNearOrEven stehen, damit das
  Argument von f2xm1 sich im erlaubten Bereich befindet }
assembler;
var
  StatusWord: word;
asm
	fld	X
	frndint
	fld	X
	fsub	st, st(1)
	ftst
	fstsw	StatusWord
	fwait
	test	StatusWord, stC0
	jz	@ge
	fabs
@ge:	f2xm1	{ FPU >= 387: -1 <= st <= 1; FPU <= 287 : 0 <= st <= 0.5 }
	fld1
	faddp	st(1), st
	jz	@ge2
	fld1
	fdivr
@ge2:	fscale	{ FPU <= 287 : Resultat undefiniert, falls 0 < st(1) < 1 }
	fstp	st(1)
	fwait
end;
(*
function exp2scale(P: byte; X: Smallint): Integer; assembler;
{ computes 2^(2^P * X/32768) * 32768 }
asm
	push	ebx
	movzx	ebx, X
	mov	al, bh
	mov	cl, 7
	sub	cl, P
	sar	al, cl
	sub	cl, 3
	shr	bx, cl
	mov	cl, al
	and	bx, 2047
	shl	bx, 1
	mov	ax, WORD PTR ExpTable[ebx]
	dec	cl
	js	@2

	mov	dx, 1			{ MSBit substituieren }
	mov	ch, 0
	jcxz	@exit
@1:	shl	ax, 1
	rcl	dx, 1
	loop	@1
	jmp	@exit

@2:	not	cl
	stc
	rcr	ax, 1
	shr	ax, cl
	adc	ax, 0			{ rounding }
	mov	dx, 0
@exit:	pop	ebx
end;
*)
{$IFDEF NoMath}

function Ceil(X: Extended): Integer;
begin
  Result := Trunc(X);
  if Frac(X) > 0 then
    Inc(Result);
end;

function Floor(X: Extended): Integer;
begin
  Result := Trunc(X);
  if Frac(X) < 0 then
    Dec(Result);
end;

function Log10(X: Extended): Extended;
asm
	FLDLG2
	FLD     X
	FYL2X
        FWAIT
end;

function Log2(X: Extended): Extended;
asm
        FLD1
        FLD     X
        FYL2X
        FWAIT
end;
{$ENDIF}
function Factorial(X: word): extended;
{ computes X! }
var
  i: word;
begin
  Result := 1;
  for i := 1 to X do Result := Result*i;
end;

procedure FPU_RoundMethod(Method: word);	assembler;
var
  FPU: record
    ControlWord: word;
  end;
asm
	fclex				{ clear pending exceptions }
	fstcw	FPU.ControlWord;
	mov	ax, FPU.ControlWord;
	and	ax, not RC_Mask
	or	ax, Method
	mov	FPU.ControlWord, ax
	fldcw	FPU.ControlWord
end;

function I0(X: single): single;
{ berechnet modifizierte Besselfunktion 1. Gattung, 0. Ordnung.
  Quelle: Oppenheim / Schafer: Zeitdiskrete Signalverarbeitung,
  7.4.3 (S. 519), Oldenburg 1992 }
var
  d, dS, S: single;
begin
  S  := 1.0;
  dS := 1.0;
  d  := 0.0;
  repeat
    d := d + 2.0;
    dS := dS * Sqr(X)/Sqr(d);
    S := S + dS;
  until dS < S*0.2E-8;
  I0 := S;
end;

function Interpolate(X, X1, Y1, X2, Y2: Integer): Integer;
begin
  Interpolate := Y1 + (X-X1) * (Y2-Y1) div (X2-X1);
end;

function Interpolate_F(X, X1, Y1, X2, Y2: Single): Single;
assembler;
{ Interpolate_F := y1 + (y2 - y1) * (x - x1) / (x2 - x1); }
asm
	fld	Y1
	fld	Y2
	fsub	st, st(1)
	fld	X
	fsub	X1
	fmul
	fld	X2
	fsub	X1
	fdiv
	fadd
	fwait
end;

function MaxS(Arg1, Arg2: Single): Single;
begin
  if Arg1 < Arg2
    then Result := Arg2
    else Result := Arg1;
end;

function MaxD(Arg1, Arg2: Double): Double;
begin
  if Arg1 < Arg2
    then Result := Arg2
    else Result := Arg1;
end;

function Max(Arg1, Arg2: Extended): Extended;
begin
  if Arg1 < Arg2
    then Result := Arg2
    else Result := Arg1;
end;

function MinS(Arg1, Arg2: Single): Single;
begin
  if Arg1 > Arg2
    then Result := Arg2
    else Result := Arg1;
end;

function MinD(Arg1, Arg2: Double): Double;
begin
  if Arg1 > Arg2
    then Result := Arg2
    else Result := Arg1;
end;

function Min(Arg1, Arg2: Extended): Extended;
begin
  if Arg1 > Arg2
    then Result := Arg2
    else Result := Arg1;
end;

function NextMultipleTo(X, Y: Integer): Integer;
assembler;
asm
	mov	ecx, edx
	xor	ecx, eax
	mov	ecx, edx		// ecx <- Y
	jns	@1
	neg	edx
@1:	sar	edx, 1
	jns	@2
	adc	edx, 0
@2:	add	eax, edx		// add X, dx
	cwd
	push	eax
	idiv	ecx		// idiv Y
	pop	eax
	sub	eax, edx
end;

function NormDistribution(rx, ru, rs: single): single;
{ ========================================
  aus c't 2/92, berechnet Normalverteilung
  ======================================== }
const
  Epsilon = 1E-8;
  Itermax = 100;
var
  l: integer;
  zws, zwp, zw: double;
begin
  zwp := rx;
  zw := 0;
  l := 0;
  Result := 1;
  if rs <> 0 then
  begin
    rx := (rx-ru)/rs;
    if rx <-10 then Result := 0 else
    if rx <= 10 then
    begin
      repeat Inc(l);
	zwp := zwp * (Sqr(rx)/(l*2));
	zws := zwp/Succ(2*l);
	if Odd(l) then zw := zw-zws else zw := zw+zws;
      until (Abs(zws) < Epsilon) or (l=IterMax);
      if l < IterMax then zw := (zw+rx) * divSqrt2Pi + 0.5;
      if Abs(zw)   < Epsilon then Result := 0 else
      if Abs(1-zw) < Epsilon then Result := 1 else
      Result := zw;
    end;
  end;
end;

function Long(X: single): Longint;	assembler;
{ Long := Round(Min(Max(X, LongMin), LongMax) }
asm
{ Die Mantisse von single ist bloss 23 bit lang, so dass
  Werte >= $7FFFFFC0 auf $80000000 gerundet werden.
  Der groesste Wert, der nicht zum Runtime Error 207
  "Invalid floating point operation" fuehrt, wird auf
  $7FFFFF80 gerundet. }
	{ >= $7FFFFFBF ? [longint(single = 2^31) = $4F000000] }
	cmp	LongRec(X).Hi, $4F00
	jge	@Max
	jns	@1
	cmp	LongRec(X).Hi, $CF00
	jae	@Min
@1:	fld	X
	fistp	Result
	fwait
	mov	eax, Result
	jmp	@exit

@Min:	mov	eax, $80000000
	jmp	@exit

@Max:	mov	eax, $FFFFFFFF
@exit:
end;

function Sign(X: Integer): Integer;
asm
	or	eax, eax
	je	@exit
	js	@Minus
	mov	eax, 1
	ret
@Minus:	mov	eax, -1
@exit:
end;

function SqrFixed(X: TFixed): TFixed;
{ Result in edx:eax }
asm
	imul	eax
	shrd	eax, edx, 16
	sar	edx, 16
end;

end.
