(************************************************************************
 Author:	Robert Rossmair
		Wagelwitz, Rodaer Str. 6
		04688 Mutzschen
		Germany

	e-mail:	Robert.Rossmair@t-online.de
	http://home.t-online.de/home/Robert.Rossmair/

 Module:	ZoomMain

 Version:	1.7.3, 21-May-1999

 Compatibility:	Delphi 2.0 & 3

 Description:   Signal display component demo program 

 Copyright  1998  Robert Rossmair

 Use, modification and distribution is allowed without limitation,
 warranty, or liability of any kind.
 ************************************************************************)
unit ZoomMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, MMSystem, ComCtrls, ExtCtrls, MPlayer, Buttons, RRScroll,
  RRMath, StdCtrls, RRColors, ZoomDisp;

type
  TSignalDisplay = class(TForm)
    OpenDialog: TOpenDialog;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Open1: TMenuItem;
    Zoom1: TMenuItem;
    ZoomOff1: TMenuItem;
    ZoomIn1: TMenuItem;
    ZoomOut1: TMenuItem;
    StatusBar: TStatusBar;
    SaveSignalImage: TMenuItem;
    SaveDialog: TSaveDialog;
    Markers1: TMenuItem;
    AlignToTop1: TMenuItem;
    AlignToBottom1: TMenuItem;
    PaintMode1: TMenuItem;
    Hollow1: TMenuItem;
    Solid1: TMenuItem;
    Panel1: TPanel;
    MediaPlayer1: TMediaPlayer;
    Timer1: TTimer;
    ZoomDisplay1: TZoomDisplay;
    Scales1: TMenuItem;
    Horizontal1: TMenuItem;
    Vertical1: TMenuItem;
    SpeedButton1: TSpeedButton;
    TrackBar1: TTrackBar;
    WindowHeightScrollBar: TRRScrollBar;
    Panel2: TPanel;
    WindowWidthScrollBar: TRRScrollBar;
    Panel3: TPanel;
    Grid1: TMenuItem;
    HorizontalGridItem: TMenuItem;
    VerticalGridItem: TMenuItem;
    EditMenu: TMenuItem;
    SelectionColor1: TMenuItem;
    ColorDialog: TColorDialog;
    Color1: TMenuItem;
    ColorLuminance: TScrollBar;
    SelectionColorclDefault1: TMenuItem;
    procedure Open1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ZoomDisplay1Paint(Sender: TObject);
    procedure ZoomOff1Click(Sender: TObject);
    procedure ZoomOut1Click(Sender: TObject);
    procedure ZoomIn1Click(Sender: TObject);
    procedure ZoomDisplayWorldPosChange(Sender: TObject; X, Y: Double);
    procedure SaveSignalImageClick(Sender: TObject);
    procedure SaveDialogTypeChange(Sender: TObject);
    procedure ZoomDisplay1ZoomMax(Sender: TObject);
    procedure ZoomDisplay1ZoomMin(Sender: TObject);
    procedure ZoomDisplay1DragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure AlignToTop1Click(Sender: TObject);
    procedure AlignToBottom1Click(Sender: TObject);
    procedure ZoomDisplay1SelectionChange(Sender: TObject);
    procedure ZoomDisplay1MouseLeave(Sender: TObject);
    procedure Hollow1Click(Sender: TObject);
    procedure Solid1Click(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure MediaPlayer1Click(Sender: TObject; Button: TMPBtnType;
      var DoDefault: Boolean);
    procedure MediaPlayer1Notify(Sender: TObject);
    procedure Vertical1Click(Sender: TObject);
    procedure Horizontal1Click(Sender: TObject);
    procedure ZoomDisplay1GetHorzScalePoint(Sender: TObject;
      var Value: Extended);
    procedure ZoomDisplay1GetVertScalePoint(Sender: TObject;
      var Value: Extended);
    procedure SpeedButton1Click(Sender: TObject);
    procedure TrackBar1Change(Sender: TObject);
    procedure ZoomDisplay1WindowChange(Sender: TObject);
    procedure WindowWidthScrollBarChange(Sender: TObject);
    procedure WindowHeightScrollBarChange(Sender: TObject);
    procedure VerticalGridItemClick(Sender: TObject);
    procedure HorizontalGridItemClick(Sender: TObject);
    procedure SelectionColor1Click(Sender: TObject);
    procedure Color1Click(Sender: TObject);
    procedure ColorLuminanceChange(Sender: TObject);
    procedure SelectionColorclDefault1Click(Sender: TObject);
  private
    Buf: PByteArray;
    FBitsPerSample: Integer;
    FChannels: Integer;
    FSampleRate: Integer;
    FPlayPos: Integer;
    FMark,
    FSelStart,
    FSelEnd: THorzMarker;
    FImageFileName: string;
    FHLS: THLSVector;
    FMonochrome: Boolean;
    procedure ChangeSelection(Sender: TObject; var Pos: Double);
    procedure OpenFile(FileName: string);
    function Percent(Value: Extended): Extended;
    procedure ReadWavFile(FileName: string);
  end;

var
  SignalDisplay: TSignalDisplay;

implementation

{$R *.DFM}

const
{$IFDEF VER90}
  SFileNotFound = 'File not found';
{$ENDIF}
  Wave_Format_ADPCM = 2;
  MSADPCM_NUM_COEF = 7;  { ADPCM: number of predictors }
type
  ArrayIndex = 0..99999999;
  TByteArray = array[ArrayIndex] of Byte;
  PByteArray = ^TByteArray;
  TSmallintArray = array[ArrayIndex] of Smallint;
  PSmallintArray = ^TSmallintArray;
  { Predictors for ADPCM }
  TADPCMCoef = packed record
    Coef1, Coef2: SmallInt;
  end;
  TADPCMCoefSet = Array[0..MSADPCM_NUM_COEF-1] of TADPCMCoef;
  PADPCMCoefSet = ^TADPCMCoefSet;

  TWaveFormatEx = packed record
    wFormatTag: Word;         { format type }
    nChannels: Word;          { number of channels (i.e. mono, stereo, etc.) }
    nSamplesPerSec: Longint;  { sample rate }
    nAvgBytesPerSec: Longint; { for buffer estimation }
    nBlockAlign: Word;        { block size of data }
    wBitsPerSample: Word;
    cbSize: Word;  { SizeOf(wSamplesPerBlock..Predictors. Optional with PCM }
    wSamplesPerBlock: SmallInt;
    wNumCoef: SmallInt;  { const 7 }
    CoefSet: TADPCMCoefSet;
  end;
  PWaveFormatEx = ^TWaveFormatEx;

procedure Error(Text: string);
var
  StackTop: record end;
  Stack: record
    EBP: Integer;
    ReturnAddress: Pointer;
  end absolute StackTop;
begin
  raise Exception.Create(Text) at Stack.ReturnAddress;
end;

procedure TSignalDisplay.ReadWavFile(FileName: string);
var
  mmioHandle: Integer;
  SampleCount, DataBytes: LongInt;
  SrcFormat: TWaveFormatEx;
  ckRIFF, ck: TMMCKINFO;

  procedure ErrExit(Msg: String);
  begin
    Error(FileName+': '+Msg);
  end;

begin
  FreeMem(Buf);
  Buf := nil;
  if (FileName = '') or not FileExists(FileName)
    then Error('File '+FileName+' not found.');
  mmioHandle := mmioOpen(PChar(FileName), nil, MMIO_READ or MMIO_ALLOCBUF);
  if mmioHandle = 0 then ErrExit('mmioOpen failed');
  try
    ckRIFF.fccType := mmioStringToFourCC('WAVE',0);
    if mmioDescend(mmioHandle, @ckRIFF, nil, MMIO_FINDRIFF) <> 0
      then ErrExit('WAVE chunk not found');

    FillChar(SrcFormat,SizeOf(SrcFormat), 0);
    SampleCount := 0;

    while mmioDescend(mmioHandle, @ck, @ckRIFF, 0) = MMSYSERR_NOERROR do
    begin
      if (ck.dwDataOffset + ck.cksize) > (ckRIFF.dwDataOffset + ckRIFF.cksize)
	then ErrExit('Chunk size mismatch');
      if ck.ckid = mmioStringToFourCC('fmt ',0) then
      begin
	if SrcFormat.wFormatTag = 0 then
	begin
	  if ck.ckSize > SizeOf(TWaveFormatEx)
	     then ErrExit('ADPCM, unexpected number of coefficients');
	  if mmioRead(mmioHandle, @SrcFormat, ck.ckSize) <> ck.ckSize
	     then ErrExit('Error reading ''fmt '' chunk');
	end;
      end else if ck.ckid = mmioStringToFourCC('fact',0) then
      begin
	if (SampleCount = 0) and (SrcFormat.wFormatTag = Wave_Format_ADPCM) then
	begin
	  mmioRead(mmioHandle, @SampleCount, SizeOf(LongInt));
	end; { else multiple fact chunks - ignore }
      end else if ck.ckid = mmioStringToFourCC('data',0) then
      begin
	DataBytes := ck.cksize;
	case SrcFormat.wFormatTag of
	  0: ErrExit('No ''fmt '' chunk');
	  Wave_Format_PCM:
	    with SrcFormat do
	      { not documented: Avg div SamplesPerSec always integer }
	      SampleCount := DataBytes div (nAvgBytesPerSec div nSamplesPerSec);
	  Wave_Format_ADPCM: ;
	 else
	  ErrExit('''fmt '' chunk exists, but unknown format');
	end;
	GetMem(Buf, DataBytes);
	mmioRead(mmioHandle, PChar(Buf), DataBytes);
	FChannels := SrcFormat.nChannels;
	FBitsPerSample := SrcFormat.wBitsPerSample;
	FSampleRate := SrcFormat.nSamplesPerSec;
	with ZoomDisplay1 do
	begin
	  case FBitsPerSample of
	     8: World := Rect(0, High(Byte), SampleCount-1, Low(Byte));
	    16: World := Rect(0, High(Smallint), SampleCount-1, Low(Smallint));
	    24: World := Rect(0, High24BitSample, SampleCount-1, Low24BitSample);
	  end;
	  MaxWindowWidth := SampleCount;
	  MaxWindowHeight := WorldHeight;
          with WindowWidthScrollBar do
          begin
            Max := MaxWindowWidth;
            Min := MinWindowWidth;
            LargeChange := Max div 32;
            if LargeChange < SmallChange * 2 then LargeChange := SmallChange * 2;
          end;
          with WindowHeightScrollBar do
          begin
            Max := MaxWindowHeight;
            Min := MinWindowHeight;
            SmallChange := Max div 128;
            LargeChange := Max div 32;
          end;
	  Window := World;
	  { need 1 + FChannel colors to make
	  channels distinguishable by colors: }
	  MonochromeBitmap := FMonochrome and (FChannels = 1);
          Mediaplayer1.FileName := FileName;
          Mediaplayer1.Open;
          Mediaplayer1.TimeFormat := tfSamples;
	  FSelStart.Position := 0; // reset selection start
	  FSelEnd.Position := 0;   // reset selection end
          FMark.Position := WindowWidth div 2;
	  InvalidateWorldWindow;
	end;
      end;
      { else leave subchunk }
      mmioAscend(mmioHandle, @ck, 0);
    end;  { while mmioDescend }
  finally
    mmioClose(mmioHandle,0);
  end;
end;

procedure TSignalDisplay.Open1Click(Sender: TObject);
begin
  with OpenDialog do
  if Execute then
  if ofExtensionDifferent in Options
    then Options := Options - [ofExtensionDifferent]
    else OpenFile(FileName);
end;

procedure TSignalDisplay.OpenFile(FileName: string);
begin
  if not FileExists(FileName) then Error(SFileNotFound);
  Screen.Cursor := crHourglass;
  try
    ReadWavFile(FileName);
  finally
    Screen.Cursor := crDefault;
  end;
  OpenDialog.InitialDir := ExtractFileDir(FileName);
  Caption := Format('Signal Display Demo - %s', [FileName]);
  FImageFileName := ExtractFileName(FileName);
  SetLength(FImageFileName, Length(FImageFileName)-4); // strip '.wav'
end;

procedure TSignalDisplay.FormCreate(Sender: TObject);
var
  WindowsDir: string;
begin
  FHLS := RGBtoHLS(ColorToRGB(ZoomDisplay1.Color));
  ColorLuminance.Position := FHLS.Luminance;
  FPlayPos := -1;
  FSampleRate := 44100;
  FBitsPerSample := 16;
  SetLength(WindowsDir, 1024);
  SetLength(WindowsDir, GetWindowsDirectory(PChar(WindowsDir) , 1024));
  OpenDialog.InitialDir := WindowsDir;
  FMonochrome := ZoomDisplay1.MonochromeBitmap;
  FSelStart := THorzMarker.Create(ZoomDisplay1);
  with FSelStart do
  begin
    Color := $A4FF59;
    Kind := mkRangeStart;
    Hint := 'SelectionStart = 0';
    OnPosChange := ChangeSelection;
    ShowPosition := False;
  end;
  FSelEnd := THorzMarker.Create(ZoomDisplay1);
  with FSelEnd do
  begin
    Color := FSelStart.Color;
    Kind := mkRangeEnd;
    Hint := 'SelectionEnd = 0';
    OnPosChange := ChangeSelection;
    ShowPosition := False;
  end;
  FMark := THorzMarker.Create(ZoomDisplay1);
  FMark.Visible := False;
  FMark.Color := $E8DC00;
  if ParamCount > 0 then OpenFile(ParamStr(1));
end;

procedure TSignalDisplay.FormDestroy(Sender: TObject);
begin
  FreeMem(Buf);
end;

procedure TSignalDisplay.ChangeSelection(Sender: TObject; var Pos: Double);
begin
  with ZoomDisplay1 do
  if Sender = FSelStart
    then SelectionStart := Round(Pos)
    else SelectionEnd := Round(Pos);
end;

procedure TSignalDisplay.ZoomDisplay1SelectionChange(Sender: TObject);
begin
  with ZoomDisplay1 do
  begin
    FSelStart.Position := SelectionStart;
    FSelStart.Hint := Format('SelectionStart = %d', [SelectionStart]);
    FSelEnd.Position := SelectionEnd;
    FSelEnd.Hint := Format('SelectionEnd = %d', [SelectionEnd]);
  end;
end;

procedure TSignalDisplay.ZoomDisplay1Paint(Sender: TObject);

  procedure PlotChannel(Channel: Integer);
  begin
    with ZoomDisplay1 do
    case FBitsPerSample of
       8: PlotByteArray(Slice(Buf^, WorldWidth * FChannels), WindowLeft, FChannels, Channel);
      16: PlotSmallintArray(Slice(PSmallintArray(Buf)^, WorldWidth * FChannels), WindowLeft, FChannels, Channel);
      24: Plot24BitSamples(Slice(P24BitSampleArray(Buf)^, WorldWidth * FChannels), WindowLeft, FChannels, Channel);
    end;
  end;

var
  Y: Integer;
  CPen: TPen;
  OldPen: TPen;
begin
  if Buf <> nil then
  with ZoomDisplay1 do
  begin
    OldPen := TPen.Create;
    try
      CPen := WaveformCanvas.Pen;
      OldPen.Assign(CPen);
      with Viewport do
      if (PaintMode = spHollow)
      or (Right-Left > WindowWidth) then
      // draw zero amplitude line
      if not VertScale.ShowGrid then
      begin
        CPen.Style := psDot;
        CPen.Color := clBlack;
        try
          if FBitsPerSample = 8
            then Y := 128
            else Y := 0;
          MoveTo(Window.Left, Y);
          LineTo(Window.Right, Y);
        finally
          CPen.Assign(OldPen);
        end;
      end;
      PlotChannel(0);
      if FChannels > 1 then
      begin
	CPen.Mode := pmXOR;
	// right channel: red lines
	CPen.Color := clRed xor ColorToRGB(Color);
        PlotChannel(1);
      end;
      CPen.Assign(OldPen);
    finally
      OldPen.Free;
    end;
  end;
end;

procedure TSignalDisplay.ZoomOff1Click(Sender: TObject);
begin
  ZoomOff1.Checked := True;
  ZoomDisplay1.ZoomMode := zmZoomOff;
end;

procedure TSignalDisplay.ZoomOut1Click(Sender: TObject);
begin
  ZoomOut1.Checked := True;
  ZoomDisplay1.ZoomMode := zmZoomOut;
end;

procedure TSignalDisplay.ZoomIn1Click(Sender: TObject);
begin
  ZoomIn1.Checked := True;
  ZoomDisplay1.ZoomMode := zmZoomIn;
end;

function TSignalDisplay.Percent(Value: Extended): Extended;
begin
  Result := 100*(2*Value+1)/((1 shl FBitsPerSample)-1);
end;

procedure TSignalDisplay.ZoomDisplayWorldPosChange(Sender: TObject; X, Y: Double);
begin
  StatusBar.Panels[0].Text := Format('sample: %d (%f msec)', [Round(X), X*1000/FSampleRate]);
  if FBitsPerSample = 8 then Y := Y - 128;
  StatusBar.Panels[1].Text := Format('amplitude: %f %%', [Percent(Y)]);
end;

procedure TSignalDisplay.SaveSignalImageClick(Sender: TObject);
const
  Extension: array[TImageFileFormat] of string[4] = ('.bmp', '.emf', '.wmf');
var
  Ext: string[4];
begin
  with SaveDialog do
  begin
    DefaultExt := 'bmp';
    FileName := FImageFileName;
    if Execute then
    begin
      Ext := LowerCase(ExtractFileExt(FileName));
      with ZoomDisplay1 do
      begin
	if Ext = '.bmp' then ImageFileFormat := imBMP else
	if Ext = '.emf' then ImageFileFormat := imEMF else
	if Ext = '.wmf' then ImageFileFormat := imWMF else
	begin
	  ImageFileFormat := TImageFileFormat(Ord(FilterIndex)-1);
	  FileName := FileName + Extension[ImageFileFormat];
	end;
        FImageFileName := FileName;
	ZoomDisplay1.SaveToFile(FileName);
      end;
      InitialDir := ExtractFileDir(FileName);
    end;
  end;
end;

procedure TSignalDisplay.SaveDialogTypeChange(Sender: TObject);
const
  Ext: array[1..3] of string[3] = ('bmp', 'emf', 'wmf');
begin
  with SaveDialog do DefaultExt := Ext[FilterIndex];
end;

procedure TSignalDisplay.ZoomDisplay1ZoomMax(Sender: TObject);
begin
  ZoomDisplay1.ZoomMode := zmZoomOut;
  ZoomOut1.Checked := True;
end;

procedure TSignalDisplay.ZoomDisplay1ZoomMin(Sender: TObject);
begin
  ZoomDisplay1.ZoomMode := zmZoomIn;
  ZoomIn1.Checked := True;
end;

procedure TSignalDisplay.ZoomDisplay1DragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
var
  M: TMarker absolute Source;
begin
  Accept := (Source = FSelStart)
         or (Source = FSelEnd)
         or (Source = FMark);
end;

procedure TSignalDisplay.AlignToTop1Click(Sender: TObject);
begin
  AlignToTop1.Checked := True;
  FSelStart.Align := alTop;
  FSelEnd.Align := alTop;
  FMark.Align := alTop;
end;

procedure TSignalDisplay.AlignToBottom1Click(Sender: TObject);
begin
  AlignToBottom1.Checked := True;
  FSelStart.Align := alBottom;
  FSelEnd.Align := alBottom;
  FMark.Align := alBottom;
end;

procedure TSignalDisplay.ZoomDisplay1MouseLeave(Sender: TObject);
begin
  StatusBar.Panels[0].Text := '';
  StatusBar.Panels[1].Text := '';
end;

procedure TSignalDisplay.Hollow1Click(Sender: TObject);
begin
  ZoomDisplay1.PaintMode := spHollow;
  Hollow1.Checked := True;
end;

procedure TSignalDisplay.Solid1Click(Sender: TObject);
begin
  ZoomDisplay1.PaintMode := spSolid;
  Solid1.Checked := True;
end;

procedure TSignalDisplay.Timer1Timer(Sender: TObject);
var
  X: Integer;
begin
  X := Mediaplayer1.Position;
  if X = ZoomDisplay1.WorldWidth then
  begin
    Mediaplayer1.Notify := True;
    Mediaplayer1.Stop;
    Timer1.Enabled := False;
  end;
  ZoomDisplay1.HighlightedSample := X;
end;

procedure TSignalDisplay.MediaPlayer1Click(Sender: TObject;
  Button: TMPBtnType; var DoDefault: Boolean);
begin
  with Mediaplayer1 do
  case Button of
    btPlay: Timer1.Enabled := True;
    btPause: Timer1.Enabled := Mode = mpPaused;
    btStop: Notify := True;
  end;
end;

procedure TSignalDisplay.MediaPlayer1Notify(Sender: TObject);
begin
  if Mediaplayer1.Mode = mpStopped then
  begin
    Timer1.Enabled := False;
    ZoomDisplay1.HideHighlightedSample;
  end;
end;

procedure TSignalDisplay.Vertical1Click(Sender: TObject);
begin
  Vertical1.Checked := not Vertical1.Checked;
  ZoomDisplay1.VertScale.Visible := Vertical1.Checked;
end;

procedure TSignalDisplay.Horizontal1Click(Sender: TObject);
begin
  Horizontal1.Checked := not Horizontal1.Checked;
  ZoomDisplay1.HorzScale.Visible := Horizontal1.Checked;
end;

procedure TSignalDisplay.ZoomDisplay1GetHorzScalePoint(Sender: TObject;
  var Value: Extended);
begin
  Value := Value / FSampleRate;
end;

procedure TSignalDisplay.ZoomDisplay1GetVertScalePoint(Sender: TObject;
  var Value: Extended);
begin
  if FBitsPerSample = 8 then Value := Value - 128;
  Value := Percent(Value);
end;

procedure TSignalDisplay.SpeedButton1Click(Sender: TObject);
begin
  FMark.Visible := SpeedButton1.Down;
end;

procedure TSignalDisplay.TrackBar1Change(Sender: TObject);
begin
  ZoomDisplay1.MinVertScaleWidth := TrackBar1.Position;
end;

procedure TSignalDisplay.ZoomDisplay1WindowChange(Sender: TObject);
begin
  WindowWidthScrollBar.Position := ZoomDisplay1.WindowWidth;
  WindowHeightScrollBar.Position := ZoomDisplay1.WindowHeight;
end;

procedure TSignalDisplay.WindowWidthScrollBarChange(Sender: TObject);
var
  W: Integer;
begin
  W := WindowWidthScrollBar.Position;
  with ZoomDisplay1 do
  begin
    WindowLeft := WindowLeft+(WindowWidth-W) div 2;
    WindowWidth := W;
  end;
end;

procedure TSignalDisplay.WindowHeightScrollBarChange(Sender: TObject);
var
  W: TRect;
  H: Integer;
begin
  H := WindowHeightScrollBar.Position;
  with ZoomDisplay1 do
  begin
    W := Window;
    Inc(W.Top, (H-WindowHeight) div 2);
    W.Bottom := W.Top-H+1;
    Window := W;
  end;
end;

procedure TSignalDisplay.VerticalGridItemClick(Sender: TObject);
begin
  VerticalGridItem.Checked := not VerticalGridItem.Checked;
  ZoomDisplay1.HorzScale.ShowGrid := VerticalGridItem.Checked;
end;

procedure TSignalDisplay.HorizontalGridItemClick(Sender: TObject);
begin
  HorizontalGridItem.Checked := not HorizontalGridItem.Checked;
  ZoomDisplay1.VertScale.ShowGrid := HorizontalGridItem.Checked;
end;

procedure TSignalDisplay.SelectionColor1Click(Sender: TObject);
begin
  with ColorDialog do
  begin
    Color := ZoomDisplay1.SelectionColor;
    if Execute then
    begin
      ZoomDisplay1.SelectionColor := Color;
      with SelectionColorclDefault1 do
      begin
        Checked := False;
        Enabled := True;
      end;
    end;
  end;
end;

procedure TSignalDisplay.Color1Click(Sender: TObject);
begin
  with ColorDialog do
  begin
    Color := ZoomDisplay1.Color;
    if Execute then
    begin
      ZoomDisplay1.Color := Color;
      FHLS := RGBtoHLS(ColorToRGB(Color));
      ColorLuminance.Position := FHLS.Luminance;
    end;
  end;
end;

procedure TSignalDisplay.ColorLuminanceChange(Sender: TObject);
begin
  with FHLS do
    ZoomDisplay1.Color := HLStoRGB(Hue, ColorLuminance.Position, Saturation);
end;

procedure TSignalDisplay.SelectionColorclDefault1Click(Sender: TObject);
begin
  SelectionColorclDefault1.Checked := True;
  SelectionColorclDefault1.Enabled := False;
  ZoomDisplay1.SelectionColor := clDefault;
end;

end.
